#! /usr/bin/env bash
##########################################################################
#                                                                        #
#  This file is part of Frama-C.                                         #
#                                                                        #
#  Copyright (C) 2007-2024                                               #
#    CEA (Commissariat à l'énergie atomique et aux énergies              #
#         alternatives)                                                  #
#                                                                        #
#  you can redistribute it and/or modify it under the terms of the GNU   #
#  Lesser General Public License as published by the Free Software       #
#  Foundation, version 2.1.                                              #
#                                                                        #
#  It is distributed in the hope that it will be useful,                 #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#  GNU Lesser General Public License for more details.                   #
#                                                                        #
#  See the GNU Lesser General Public License version 2.1                 #
#  for more details (enclosed in the file licenses/LGPLv2.1).            #
#                                                                        #
##########################################################################

# Note:
#
# While this script can be run locally, it is meant to run in the Frama-C CI.
# Thus, it expects to be run from the root of the Frama-C directory and that
# some CI artifacts are available. Namely:
#   - the 'api-json-doc' as generated by the nix target api-json-doc
# Availability of the files is NOT checked when the script starts.
# The script:
#   - clones the private JSON doc repository
#   - remove all files
#   - copies the content of the api-json-doc artifact to the clone
#   - commits the changes
#   - pushes the branch

##########################################################################

JSON_API_GIT="git@git.frama-c.com:frama-c/frama-c-api-json.git"
JSON_API_DIR="frama-c-api-json"
ARTIFACT="./api/frama-c-api-json.tar.gz"

echo "$FRAMA_CI_BOT_SSH_PRIVATE" | base64 -d > nix/frama-c-public/id_ed25519
chmod 400 nix/frama-c-public/id_ed25519

if ! git clone $JSON_API_GIT $JSON_API_DIR
then
  echo "Failed to clone JSON API"
  exit 128
fi

rm -rf $JSON_API_DIR/frama-c* $JSON_API_DIR/qed $JSON_API_DIR/index.html.json
tar xzf $ARTIFACT $JSON_API_DIR
chown root $JSON_API_DIR

cd $JSON_API_DIR

git config user.name  "Frama-CI Bot"
git config user.email "frama-ci-bot@frama-c.com"

function add {
  git add -A
  if [ "$?" -ne "0" ]; then
    echo "Failed to add files"
    exit 128
  fi
}

function commit {
  git commit -F- <<EOF
Update with last Frama-C master
EOF
  if [ "$?" -ne "0" ]; then
    echo "Failed to commit on branch"
    exit 128
  fi
}

function push {
  git push
  if [ "$?" -ne "0" ]; then
    echo "Failed to push branch"
    exit 128
  fi
}

add

git diff --cached --exit-code > /dev/null
if [ "$?" -ne "0" ]; then
  commit
  push
fi
